package Extract_genome_info;

use strict;
use warnings;

sub file{
	my($input_file,$output_file) = @_;
	my(%genome) = \hash($input_file);
	
	unless(open(OUT,">$output_file")){
		print STDERR "Can not open file $output_file!\n";
	}

	foreach my $gene(split(/ /,$genome{genes})){
		print OUT ">$gene\n".$genome{$gene}{aa}."\n";
	}
	
	close OUT;
}

sub hash{
	my($file) = @_;
	my(%genome) = ();
	my($gene_frame) = 0;
	my($definition) = 0;
	my($genes) = 0;
	my($aa,$gene,$start,$end,$locus_tag);
	my($next) = 0;
	
	$genome{file} = $file;
	if($file =~ /(.+)\.(cgi|gbk)/){
		$genome{name} = $1;
		if($file =~ /(.+\/)([^\/]+)\.(cgi|gbk)/){
			$genome{directory} = $1;
			$genome{Pure_name} = $2;
		}else{
			$genome{directory} = "";
			$genome{Pure_name} = $genome{name};
		}
	}
	
	unless(open(FILE,"$file")){
		print STDERR "Can not open file $file!\n";
	}

	foreach my $line(<FILE>){
 		if($genes == 0){
			if($line =~ /^LOCUS\s+(\S+)\s+(\S+)/){
			#print "$1 $2\n";
				$genome{LOCUS} = $1;
				$genome{genes} = "";
				$genome{length} = $2;
				$genome{numberGenes} = 0;
			}elsif($line =~ /^SOURCE\s+(.+)$/){
				$genome{SOURCE} = $1;
				$definition = 1;
			}elsif($line =~ /^\s+ORGANISM/){
				$definition = 0;
			}elsif($line =~ /FEATURES/){
				$genes = 1;
			}
		}else{
			if($line =~ /CDS[^0-9]+([0-9]+).*[^0-9]{2,}([0-9]+)\){0,2}$/){
				$start = $1;
				$end = $2;
				$gene_frame = 1;
			}elsif($gene_frame == 1 && $line =~ /gene="([^"\s]+)"/){
				$gene = $1;
				if($gene !~ /unknown/){
					$gene_frame++;
				}
			}elsif($line =~ /locus_tag="([^"]+)/){
				$locus_tag = $1;
				if($gene_frame == 1){
					$gene = $locus_tag;
					$gene_frame++;
				}
			}elsif($line =~ /translation="([A-Z]+)("?)/){
				$aa = $1;
				$gene_frame++;
				if($2 =~ /\"/){
					$next = 1;
				}
			}elsif($gene_frame == 3 && $line =~ /^\s+([A-Z]*)("?)$/){
				$aa .= $1;
				if($2 =~ /\"/){
					$next = 1;
				}
			}
			
			if($gene_frame == 3 && length($aa) > 0 && $next == 1){
				$genome{genes} .= "$locus_tag ";
				$genome{allgenes} .= "$locus_tag ";
				$genome{$locus_tag}{position} = $start;
				$genome{$locus_tag}{end} = $end;
				$genome{$locus_tag}{aa} = $aa;
				$genome{$locus_tag}{gene} = $gene;
				$genome{numberGenes}++;
				$next = 0;
				$gene_frame = 0;
			}
		}
	}
	return \%genome;
}

1;