#!/usr/bin/perl -w
#requires perl modules GD, GD::SVG and Math::float to run.
#requires blastall

#Reference: Toft, C and Fares, MA (2006). GRAST: a new way of Genome Reduction Analysis using comparative genomics. Bioinformatics

use strict;
use warnings;
use lib '/windows2/Modules';
use Genome_info;
use Options;
use GeneSuccession;
use GeneMovement;

print "\e[1mReference: Toft, C and Fares, MA (2006). GRAST: a new way of Genome Reduction Analysis using comparative genomics. Bioinformatics\e[m\n";

my(@options) = @ARGV;
print "@options\n";
my($small_genome);
my($large_genome);
my($e_value,$p_e_value,$accuracy);
my($name) = 0;
my($output) = 0;
my(@temp);

my($option) = "@options";


if($option =~ /^[^\s]+\.(cgi|gbk)\s+[^\s]+\.(cgi|gbk)/){
	$small_genome = $ARGV[0];
	$large_genome = $ARGV[1];
	
	#checking that the entered files exist
	if( (!(-e $small_genome) || !(-e $large_genome))){
		print "one or both of the entered file names does not exist!\n";
		
		if($option =~ / -p| -r| -l| -b| -s| -m| -g| -c| -t| -x| -y| -j| -h| -all/){
			die;
		}else{
			get_file_names(\$small_genome,\$large_genome);
		}
	}
#die if the directory doesn't contail both of the genBank file and the user want to run the program automaticly
}elsif($option =~ / -p| -r| -l| -b| -s| -m| -g| -c| -t| -x| -y| -j| -h| -all/){
	print STDERR "one or both of the entered file names does not exist!\n";
	die;
}else{
	get_file_names(\$small_genome,\$large_genome);
}

Genome_info::get($small_genome,$large_genome);

if($option =~ /-d ([^\s]+)/){
	$Genome_info::genomes[0]{directory} = $1;
	if($Genome_info::genomes[0]{directory} !~ /\/$/){
		$Genome_info::genomes[0]{directory} = $Genome_info::genomes[0]{directory}."/";
	}
}
if(! -d $Genome_info::genomes[0]{directory}){
	mkdir($Genome_info::genomes[0]{directory});
}

my(%fileNames) = (
	'-r' =>	$Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".rearrange.svg",
	'-p' =>	$Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".common_genes.txt",
	'-l' =>	$Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".dist.txt",
	'-b' =>	$Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".dist.svg",
	'-s' =>	$Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".succession.txt",
	'-m' =>	$Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".succession.svg",
	'-g' =>	$Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".genes.svg",
	'-c' =>	$Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".common_genes.svg",
	'-t' =>	$Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".translacation.txt",
	'-x' => $Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".cgsc_stats1.svg",
	'-y' => $Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".cgsc_stats2.svg",
	'-j' => $Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".junkDNA.svg",
	'-h' => $Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".junkDNA.txt",
	'-k' =>	$Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".non-common_genes.txt",
	'-n' =>	$Genome_info::genomes[0]{directory}.$Genome_info::genomes[0]{LOCUS}."_".$Genome_info::genomes[1]{LOCUS}.".non-common_genes.svg",
	);

#sets the e-value for the BLAST searches of the genomes against each other
if($option =~ /-e (0\.[0-9]+)/){
	$e_value = $1;
}else{
	$e_value = 10; #$e_value = 0.000001;
}

#sets the number of simulations
if($option =~ /-a ([0-9]+)/){
	$accuracy = $1;
}else{
	$accuracy = 100;
}

#sets the e-value of the intra genomic BLAST search
if($option =~ /-o (0\.[0-9]+)/){
	$p_e_value = $1;
}else{
	$p_e_value = 0.000001; #$e_value = 0.000001;
}

if($option =~ / -f/){
	$output = 1;
}
print "The BLAST search might take a while!\n";
Genome_info::set_cor_genes($e_value);
Genome_info::remove_paralogous($p_e_value);
print "finished the BLAST searches\n";

if($option !~ / -p| -r| -l| -b| -s| -m| -g| -c| -t| -x| -y| -j| -h| -k| -n| -all/){
	manuel($e_value,\%fileNames,$output,$accuracy);
}else{
	automatic(\%fileNames,$option,$e_value,$accuracy);
}
@temp = `ls $Genome_info::genomes[0]{name}.fasta*`;
if(scalar(@temp) > 1){
	`rm $Genome_info::genomes[0]{name}.fasta.*`;
}
@temp = `ls $Genome_info::genomes[1]{name}.fasta*`;
if(scalar(@temp) > 1){
	`rm $Genome_info::genomes[1]{name}.fasta.*`;
}
print "finished\n\n";
exit;
########################################################################################################################

sub manuel{
	my($e_value,$fileNames,$ownFileName,$accuracy) = @_;
	my($output);
	my($option);
	my($options) = "";
	
	my(%hash) = (
		1  =>	'-g',
		2  =>	'-c',
		3  =>	'-r',
		4  =>	'-n',
		5  =>	'-s',
		6  =>	'-m',
		7  =>	'-y',
		8  =>	'-x',
		9  =>	'-l',
		10 =>	'-b',
		11 =>	'-p',
		12 =>	'-k',
		13 =>	'-t',
		14 =>	'-j',
		15 =>	'-h',
		);
	
	do{
		print "\nOPTIONS:
        1       shows the placement of common/non-common genes in the two\n\t\t genomes (g)
        2       shows the placement of common genes in the two genomes (c)
        3       shows the placement of non-common genes in the two genomes (n)
        4       gene pair plot of the two genomes (r)
        5       CSGC: the start position in the two genomes, the genes names\n\t\t and the number of genes that has been lost between genes (s)
        6       shows the placement of the CGSC on the two genomes (m)
        7       overall statistics on CSGC’s (y)
        8       statistics on CSGC (x)
        9       distribution of lost genes in COG – text file (l)
        10      shows the distribution of lost genes within/in COG categories\n\t\t and the overall loss of genes (b)
        11      orthologous gene pairs – gene names + e-values (p)
        12	list of genes that dont have a homolog in the other genome (k)
        13      observed and expected (before and after genome reduction)\n\t\t on seeing gathering of genes with similar function (t)
        14      shows the distribution of junk DNA (j)
        15      distribution of junk DNA - text file (h)
        q       quit.
Pick one of the options: ";
		
		$option = <STDIN>;
		chomp $option;
		if($option =~ /^\s*([1-9]|10|11|q)\s*$/i){
		
			if($option =~ /^\s*([1-9]|10|11)\s*$/i){
				$option = $hash{$1};
				print "$option\n";
				if(($option =~ /-b|-l|-t/) && ($options !~ /-b|-l|-t/)){
					Genome_info::get_cog();
				}
				if(($option =~ /-s|-m|-x|-y/) && ($options !~ / -s| -m| -x| -y/)){
					GeneSuccession::start(\@Genome_info::genomes,\%Genome_info::cor_genes);
				}
				
				if($option =~ /(-s)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					GeneSuccession::conserved_gene_succession(\@Genome_info::genomes,\%Genome_info::cor_genes,$output);
					$options .= " -s";
				}elsif($option =~ /(-m)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					GeneSuccession::conserved_gene_succession_genome_gif(\@Genome_info::genomes,\%Genome_info::cor_genes,$output);
					$options .= " -m";
				}elsif($option =~ /(-r)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					Options::gene_rearrangement(\@Genome_info::genomes,\%Genome_info::cor_genes,$output);
					$options .= " -r";
				}elsif($option =~ /(-p)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					Options::common_genes(\@Genome_info::genomes,\%Genome_info::cor_genes,$output);
					$options .= " -p";
				}elsif($option =~ /(-l)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					Options::dist_lost_gene_function(\@Genome_info::genomes,\%Genome_info::cog,\%Genome_info::cor_genes,$output);
					$options .= " -l";
				}elsif($option =~ /(-b)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					Options::dist_lost_gene_function_gif(\@Genome_info::genomes,\%Genome_info::cog,\%Genome_info::cor_genes,$output);
					$options .= " -b";
				#are genes translocated to genes with similar function or randomly?
				}elsif($option =~ /(-t)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					GeneMovement::gene_placement(\@Genome_info::genomes,\%Genome_info::cog);
					GeneMovement::translocation_to_sim_func(\@Genome_info::genomes,\%Genome_info::cog,\%Genome_info::cor_genes,$output,$accuracy);
				}elsif($option =~ /(-c)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					Options::placement_lost_retained_genes(\@Genome_info::genomes,\%Genome_info::cor_genes,$output,0);
					$options .= " -c";
				}elsif($option =~ /(-g)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					Options::placement_lost_retained_genes(\@Genome_info::genomes,\%Genome_info::cor_genes,$output,1);
					$options .= " -g";
				}elsif($option =~ /(-n)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					Options::placement_lost_retained_genes(\@Genome_info::genomes,\%Genome_info::cor_genes,$output,2);
					$options .= " -n";
				}elsif($option =~ /(-x)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					GeneSuccession::conserved_gene_succession_gif($output);
					$options .= " -x";
				}elsif($option =~ /(-y)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					GeneSuccession::conserved_gene_succession_overall_gif(\@Genome_info::genomes,$output);
					$options .= " -y";
				}elsif($option =~ /(-j)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					Options::junkDNA(\@Genome_info::genomes,\%Genome_info::cor_genes,'j',$$fileNames{'-j'});
					$options .= " -j";
				}elsif($option =~ /(-h)/){
					$output = get_output_file($1,$fileNames,$ownFileName);
					Options::junkDNA(\@Genome_info::genomes,\%Genome_info::cor_genes,'h',$$fileNames{'-h'});
					$options .= " -h";
				}
			}
		}else{
			print "That is not an valid option!\n";
		}
	}while($option !~ /^\s*q\s*$/i);
}

sub get_output_file{
	my($option,$fileNames,$ownFileName) = @_;
	my($output);
	if($ownFileName == 1){
		do{
			print "Please ender output file name: ";
			$output = <STDIN>;
			chomp $output;
		}while($output =~ /^\s$/);
	}else{
		$output = $$fileNames{$option};
	}
	return $output;
}

sub automatic{
	my($fileNames,$option,$e_value,$accuracy) = @_;
	
	if($option =~ / -b| -l| -t| -all/){
		Genome_info::get_cog();
	}
	if($option =~ / -s| -m| -x| -y| -all/){
		GeneSuccession::start(\@Genome_info::genomes,\%Genome_info::cor_genes);
	}
	
	if($option =~ / -s| -all/){
	print "-s\n";
		GeneSuccession::conserved_gene_succession(\@Genome_info::genomes,\%Genome_info::cor_genes,$$fileNames{'-s'});
	}
	if($option =~ / -m| -all/){
	print "-m\n";
		GeneSuccession::conserved_gene_succession_genome_gif(\@Genome_info::genomes,\%Genome_info::cor_genes,$$fileNames{'-m'});
	}
	if($option =~ / -x| -all/){
	print "-x\n";
		GeneSuccession::conserved_gene_succession_gif($$fileNames{'-x'});
	}
	if($option =~ / -y| -all/){
	print "-y\n";
		GeneSuccession::conserved_gene_succession_overall_gif(\@Genome_info::genomes,$$fileNames{'-y'});
	}
	if($option =~ / -r| -all/){
	print "-r\n";
		Options::gene_rearrangement(\@Genome_info::genomes,\%Genome_info::cor_genes,$$fileNames{'-r'});
	}
	if($option =~ / -p| -all/){
	print "-p\n";
		Options::common_genes(\@Genome_info::genomes,\%Genome_info::cor_genes,$$fileNames{'-p'});
	}
	if($option =~ / -l| -all/){
	print "-l\n";
		Options::dist_lost_gene_function(\@Genome_info::genomes,\%Genome_info::cog,\%Genome_info::cor_genes,$$fileNames{'-l'});
	}
	if($option =~ / -b| -all/){
	print "-b\n";
		Options::dist_lost_gene_function_gif(\@Genome_info::genomes,\%Genome_info::cog,\%Genome_info::cor_genes,$$fileNames{'-b'});
	}
	if($option =~ / -t| -all/){
	print "-t\n";
		GeneMovement::gene_placement(\@Genome_info::genomes,\%Genome_info::cog);
		GeneMovement::translocation_to_sim_func(\@Genome_info::genomes,\%Genome_info::cog,\%Genome_info::cor_genes,$$fileNames{'-t'},$accuracy);
	}
	if($option =~ / -c| -all/){
	print "-c\n";
		Options::placement_lost_retained_genes(\@Genome_info::genomes,\%Genome_info::cor_genes,$$fileNames{'-c'},0);
	}
	if($option =~ / -n| -all/){
	print "-n\n";
		Options::placement_lost_retained_genes(\@Genome_info::genomes,\%Genome_info::cor_genes,$$fileNames{'-n'},2);
	}
	if($option =~ / -g| -all/){
	print "-g\n";
		Options::placement_lost_retained_genes(\@Genome_info::genomes,\%Genome_info::cor_genes,$$fileNames{'-g'},1);
	}
	if($option =~ / -j| -all/){
	print "-j\n";
		Options::junkDNA(\@Genome_info::genomes,\%Genome_info::cor_genes,'j',$$fileNames{'-j'});
	}
	if($option =~ / -h| -all/){
	print "-h\n";
		Options::junkDNA(\@Genome_info::genomes,\%Genome_info::cor_genes,'h',$$fileNames{'-h'});
	}
	if($option =~ / -k| -all/){
	print "-k\n";
		Options::none_common_genes(\@Genome_info::genomes,\%Genome_info::cor_genes,$$fileNames{'-k'});
	}
	
}

sub get_file_names{
	my($small_genome,$large_genome) = @_;
	
	do{
		print "File containing the small genome: ";
		$$small_genome = <STDIN>;
		chomp $$small_genome;
	}while(!(-e $$large_genome));
	
	print "\n";
	
	do{
		print "File containing the large genome: ";
		$$large_genome = <STDIN>;
		chomp $$large_genome;
	}while(!(-e $$large_genome));
}

__END__

=head1 NAME

GRASTv1.pl

=head1 REFERENCE

Toft, C and Fares, MA (2006). GRAST: a new way of Genome Reduction Analysis using comparative genomics. Bioinformatics

=head1 DESCRIPTION

=head1 REQUIRES

Perl modules GD, GD::SVG and Math::float

NCBI's blastall

=head1 SYNOPSIS

=head2 command line argument

./GRASTv1.0.pl <reduced genome’s genbank file> <reference genome’s genebank file> <(output) options>

=head2 user interface

./GRAST_2.pl <reduced genome's genbank file> <reference genome’s genebank file> <options>

Giving the reduced and the reference genome genbank files as argument are optional - if not given as argument GRAST will ask for the names of the files.

=head1 OPTIONS

=over 6

=item -a <number of simulations>


default=100

The number of simulations to be performed to estimate the expected probability of seeing the seen translocations by random (before and after genome reduction)

=item -e <e-value>


default=1e-6

The cut-off e-value for the BLAST searches of the two genomes against eachother

=item -o <e-value>


default=1e-6

The cut-off e-value for the intra BLAST searches of the two genomes.

=item -d <directory>


default=current directory

The directory in which the output files are created (including the blast output files).


=back

=head1 OUTPUT OPTIONS

=over 6

=item -g

shows the placement of common/non-common genes in the two genomes

=item -c

shows the placement of common genes in the two genomes

=item -n

shows the placement of non-common genes in the two genomes

=item -r

gene pair plot of the two genomes

=item -s

CSGC: the start position in the two genomes, the genes names and the number of genes that has been lost between genes

=item -m

shows the placement of the CGSC on the two genomes

=item -y

overall statistics on CSGC’s

=item -x

statistics on CSGC

=item -l

distribution of lost genes in COG - text file

=item -b

shows the distribution of lost genes within/in COG categories and the overall loss of genes

=item -p

orthologous gene pairs - gene names + e-values

=item -k

list of genes that don’t have a ortholog in the other genome

=item -t

observed and expected (before and after genome reduction) on seeing gathering of genes with similar function 

=item -j

shows the distribution of junk DNA

=item -h

distribution of junk DNA - text file

=item -all

runs all output options

=back

=head1 SEE ALSO

Toft, C and Fares, MA (2006). GRAST: a new way of Genome Reduction Analysis using comparative genomics. Bioinformatics

=head2 and

README_grast.pdf

=cut