package GeneMovement;

use strict;
use warnings;
use Math::BigFloat;

our(@translocations);

#number of genes between two genes and how many has been lost
sub gene_placement{
	my($genomes,$cog,$cor_genes) = @_;
	
	my($tempJp) ="";
	my($tempJm) = "";
	my($numJpj) = 0;
	my($numJpi) = 0;
	my($numJmj) = 0;
	my($numJmi) = 0;
	my($gene);
	my(@geneI) = split(/ /,$$genomes[0]{genes});
	my($i);
	my($next_i);
	my($next_gene);
	
	#loop throught each of the genes in genome i.
	for($i=0;$i<(scalar(@geneI)-1);$i++){
		#find geneI in genome J - split the genome in to two (before and after match)
		if($$genomes[1]{genesI} =~ /^(.*)\b$geneI[$i]\b (.*)/){
			$tempJp = $2;	#after gene match
			$tempJm = $1;	#before gene match
			$next_i = $i+1;
			$next_gene = $geneI[$next_i];
			if($tempJp =~ /\b$next_gene\b/){
				foreach $gene(split(/ /,$tempJp)){
					if($gene eq $next_gene){
						#is true if there are genes from genome i between gene_i and $next_gene
						if($numJpi > 0){
							push(@translocations,cog_trans($cog,$$cog{0}{$geneI[$i]},$$cog{0}{$gene}));
						}
						$numJpi = 0;
						last;
					}else{
						#counts the number of genes from genome 0 there are between $gene_i and $next_gene
						if($$genomes[0]{genes} =~ /\b$gene\b/){
							$numJpi++;
						}
					}
				}
			}elsif($tempJm =~ /\b$next_gene\b/){
				foreach $gene(reverse(split(/ /,$tempJm))){
					if($gene eq $next_gene){
						if($numJmi > 0){
							push(@translocations,cog_trans($cog,$$cog{0}{$geneI[$i]},$$cog{0}{$gene}));
						}
						$numJmi = 0;
						last;
					}else{
						if($$genomes[0]{genes} =~ /\b$gene\b/){
							$numJmi++;
						}
					}
				}
			}
		}
	}
	return @translocations;
}

#are genes translocated to genes with similar function or randomly?
sub translocation_to_sim_func{
	my($genomes,$cog,$cor_genes,$output,$num) = @_;
	my(@genesI) = split(/ /,$$genomes[0]{genes});
	my(@genesJ) = split(/ /,$$genomes[1]{genes});
	my(@genesIJ) = split(/ /,$$genomes[1]{genesIonly});
	my(@temp_cog);
	my($pro);
	
	my($prob_trans_observed) = 1;
	my(@prob_trans0);
	my(@prob_trans1);
	my($prob_trans_random_before) = 0;
	my($prob_trans_random_after) = 0;
	
	my($JKL) = "JAKLB";	#Information processing
	my($D_O) = "DYVTMNZWUO";#Cellular processes and signaling
	my($C_Q) = "CGEFHIPQ";	#Metabolism
	my($RS) = "RS";		#Poorly characterized
	my($non) = "-";
	
	
	my($gene1);
	my($gene2);
	
	#set the random generater
	srand(time|$$);
	
	$prob_trans_observed = prob_trans($cog,\@translocations);
	
	#similare the rearrangement of the genome - $num times with scalar(@translocations) translocation events of genome J
	for(my($i)=0;$i<$num;$i++){
		for(my($j)=0;$j<scalar(@translocations);$j++){
			$gene1 = $genesJ[int(rand(scalar @genesJ))];
			$gene2 = $genesJ[int(rand(scalar @genesJ))];
			$temp_cog[$j] = cog_trans($cog,$$cog{1}{$gene1},$$cog{1}{$gene2});
		}
		
		$prob_trans1[$i] = prob_trans($cog,\@temp_cog);
	}
	
	#sum up probability of the rearrangement 
	foreach my $trans(@prob_trans1){
		$prob_trans_random_before += $trans;
	}
	
	#find the average of rearrangement
	$prob_trans_random_before = $prob_trans_random_before/(scalar(@prob_trans1));
	
	my($temp1,$temp2);
	#similare the rearrangement of the genome - $num times with scalar(@translocations) translocation events of 'contructed genome' (consistes of the gene the two genomes has in common but in the order of genome J)
	for(my($i)=0;$i<$num;$i++){
		for(my($j)=0;$j<scalar(@translocations);$j++){
			$gene1 = $genesIJ[int(rand(scalar @genesIJ))];
			$gene2 = $genesIJ[int(rand(scalar @genesIJ))];
			$temp1 = $$cor_genes{0}{1}{$gene1};
			$temp2 = $$cor_genes{0}{1}{$gene2};
			$temp_cog[$j] = cog_trans($cog,$$cog{1}{$temp1},$$cog{1}{$temp2});
		}
		
		$prob_trans0[$i] = prob_trans($cog,\@temp_cog);
	}
	
	foreach my $trans(@prob_trans0){
		$prob_trans_random_after += $trans;
	}
	
	$prob_trans_random_after = $prob_trans_random_after/(scalar(@prob_trans0));
	$pro = sprintf("%.2f",100*scalar(@genesIJ)/$$genomes[0]{numberGenes});
	
	unless(open(OUT,">$output")){
		print STDERR "cant open $output\n";
	}
	my($string) = "                        ";
	printf(OUT "Gathering caused by movement probability:\n\t- observed%s= %6.4e\n\t- random before genome reduction  = %6.4e\n\t- random after genome reduction   = %6.4e\n\n",$string,$prob_trans_observed,$prob_trans_random_before,$prob_trans_random_after);
	print OUT "Note that ".$pro."% of the reduced genome has a corresponding gene in the reference genome";
	close OUT;
}

#uses multinomial density function
sub prob_trans{
	my($cog,$translocation) = @_;
	
	my($JKL) = 0;
	my($D_O) = 0;
	my($C_Q) = 0;
	my($random) = 0;
	
	my($prob_trans_log,$prob_trans);
	my($bigLength) = Math::BigFloat->new(scalar(@$translocation));
	my($bigJKL,$bigD_O,$bigC_Q,$bigRandom);	
	my($bigProJKL,$bigProD_O,$bigProC_Q,$bigProRandom);	

	foreach my $trans(@$translocation){
		if($trans eq "JKL"){
			$JKL++;
		}elsif($trans eq "D_O"){
			$D_O++;
		}elsif($trans eq "C_Q"){
			$C_Q++;
		}elsif($trans eq "random"){
			$random++;
		}
	}
	$bigJKL = Math::BigFloat->new($JKL);
	$bigD_O = Math::BigFloat->new($D_O);
	$bigC_Q = Math::BigFloat->new($C_Q);
	$bigRandom = Math::BigFloat->new($random);
	
	$bigLength->bfac();
	$bigJKL->bfac();
	$bigD_O->bfac();
	$bigC_Q->bfac();
	$bigRandom->bfac();
	
	$bigLength->blog(10);
	$bigJKL->blog(10);
	$bigD_O->blog(10);
	$bigC_Q->blog(10);
	$bigRandom->blog(10);
	
	$prob_trans_log = Math::BigFloat->new($bigLength);
	$prob_trans_log->bsub($bigJKL);
	$prob_trans_log->bsub($bigD_O);
	$prob_trans_log->bsub($bigC_Q);
	$prob_trans_log->bsub($bigRandom);

	$bigProJKL = Math::BigFloat->new($$cog{JKL}{prob});
	$bigProD_O = Math::BigFloat->new($$cog{D_O}{prob});
	$bigProC_Q = Math::BigFloat->new($$cog{C_Q}{prob});
	$bigProRandom = Math::BigFloat->new($$cog{random}{prob});
	
	$bigProJKL->blog(10);
	$bigProD_O->blog(10);
	$bigProC_Q->blog(10);
	$bigProRandom->blog(10);
	
	$bigProJKL->bmul($JKL);
	$bigProD_O->bmul($D_O);
	$bigProC_Q->bmul($C_Q);
	$bigProRandom->bmul($random);
	
	$prob_trans_log += $bigProJKL + $bigProD_O + $bigProC_Q + $bigProRandom;
	
	$prob_trans = Math::BigFloat->new(10);
	$prob_trans->bpow($prob_trans_log);

	return $prob_trans;
}

sub cog_trans{
	my($cog,$cog1,$cog2) = @_;
	
	my($result) = "";
	my($prob) = 1;
	
	if($$cog{JKL}{cat} =~ /$cog1/i && $$cog{JKL}{cat} =~ /$cog2/i){
		$result = "JKL";
	}elsif($$cog{D_O}{cat} =~ /$cog1/i && $$cog{D_O}{cat} =~ /$cog2/i){
		$result = "D_O";
	}elsif($$cog{C_Q}{cat} =~ /$cog1/i && $$cog{C_Q}{cat} =~ /$cog2/i){
		$result = "C_Q";
	}else{
		$result = "random";
	}
	
	return $result;
}

1;