package GeneSuccession;

use strict;
use warnings;
use GD::SVG;
use Graph;

use constant PI => 4 * atan2(1,1);

#global arrays
our(@common_gene_patterns);
our(@re_common_gene_patterns);

#This subroutine shoud be called first to set up the global variable.
sub start{
	my($genomes,$cor_genes) = @_;
	my(@genes) = split(/ /,$$genomes[0]{genes});
	@common_gene_patterns = get_gene_pattern($genomes,$cor_genes,\@genes);
	@genes = reverse(@genes);
	@re_common_gene_patterns = get_gene_pattern($genomes,$cor_genes,\@genes);
}

#This sub will get the genes that are in succession in the two genomes and print the result to the text file $output
sub conserved_gene_succession{
	my($genomes,$cor_genes,$output) = @_;
	my(@array1) = @common_gene_patterns;
	my(@array2) = @re_common_gene_patterns;
	my(@genes);
	my($gene);
	
	unless(open(OUT,">$output")){
		print STDERR "Cant open file";
		die;
	}
	print OUT $$genomes[1]{Pure_name}."/".$$genomes[0]{Pure_name}."\n";
	#go through each of the conserved gene successionn cluster
	foreach my $element(@array1){
		if(($gene) = ($element =~ /^\s*(\S{2,})\s+/)){
			
			$element = $$genomes[1]{$gene}{position}."/".$$genomes[0]{$$cor_genes{1}{0}{$gene}}{position}." : ".$element;
			$element =~ s/\s([^\s\/\(]{2,})\s/ $1\/$$cor_genes{1}{0}{$1} /ig;
			print OUT "$element\n\n";
		}
	}
	
	#go through each of the reversede conserved gene successionn cluster
	if(scalar(@array2)>0){
		print OUT "REVERSE\n\n";
		foreach my $element(@array2){
			if(($gene) = ($element =~ /^\s*(\S{2,})\s+/)){
				$element = $$genomes[1]{$gene}{position}."/".$$genomes[0]{$$cor_genes{1}{0}{$gene}}{position}." : ".$element;
				$element =~ s/\s([^\s\/\(]{2,})\s/ $1\/$$cor_genes{1}{0}{$1} /ig;
				print OUT "$element\n\n";
			}
		}
	}
	close OUT;
}

#This subroutine find and save the conserved gene succession pressent in the two genomes.
sub get_gene_pattern{
	my($genomes,$cor_genes,$genesI_) = @_;
	my($i) = -1;
	my($numJ);
	my($tempJ);
	my($temp);
	my($gene);
	my($pattern) = "";	#temperary contains the genes in the pattern (succession) looked at
	my($in_pattern) = 0;	#0=when we are looking at genes conserved in a pattern: 1=when the genes are not conserved in a pattern.
	my(@gene_patterns);
	my($numj) = 0;
	my($corGeneI);	#holds the next corresponding gene in the small genome (gene name is from the large genome)
	my(@geneI) = @$genesI_;
	$corGeneI = next_corresponding_gene($genomes,$cor_genes,$genesI_,\$i);
	do{
		#true when genome J contains gene i
		if($$genomes[1]{genes} =~ /^(.*)(\b$corGeneI\s.*)/){
			$numJ = 1;	#the number of genes before the gene match
			$temp = $1;	#all the genes before gene match
			$tempJ = $2;	#the gene match + all the genes after the match
			$numj = 0;	#the number of genes between gene matches
			
			while($temp =~ /\w+/ig){$numJ++;}#get the number of genes before the gene match
			
			foreach $gene(split(/ /,$tempJ)){
				START:
				#true when gene i is contained in genome J
				if($i < scalar(@geneI) && $$genomes[1]{genes} =~ /\b$corGeneI\s(.*)/){
					#true when the tempJ contains gene i
					if(($i<scalar(@geneI)) && ($tempJ =~ /\b$corGeneI\s(.*)/)){
						#true when gene i isn't the last gene in the snalle genome and gene i = gene j
						if(($i < scalar(@geneI)) && ($corGeneI eq $gene)){
							$in_pattern = 1;
							if( $pattern eq ""){
								$pattern .= $corGeneI." ";
							}else{
								$pattern .= "($numj) ".$corGeneI." ";
							}
							$corGeneI = next_corresponding_gene($genomes,$cor_genes,$genesI_,\$i);
							$numj = 0;
						#true when $gene[$i] is not containd in tempJ but contained in genome J
						}elsif(exists($$cor_genes{1}{0}{$gene})){
							add_pattern(\$in_pattern,\$pattern,\@gene_patterns,\$numJ);
							last;
						#true when gene i is contained in tempJ but is not equal to j
						}else{
							$numj++;
						}
					#true when tempJ doesnt contain gene i
					}else{
						add_pattern(\$in_pattern,\$pattern,\@gene_patterns,\$numJ);
						last;
					}
				#true when the gene i is NOT contained genome J - increment i & goto START
				}elsif($i < scalar(@geneI)){
					$corGeneI = next_corresponding_gene($genomes,$cor_genes,$genesI_,\$i);
					goto START;
				}
			}
		#when gene i is not contained in genome J
		}else{
			add_pattern(\$in_pattern,\$pattern,\@gene_patterns,\$numJ);
			$corGeneI = next_corresponding_gene($genomes,$cor_genes,$genesI_,\$i);
		}
	}while($i<scalar(@geneI));
	add_pattern(\$in_pattern,\$pattern,\@gene_patterns,\$numJ);
	
	return @gene_patterns;
}

#sub used in get_gene_pattern
#adds a succession of genes in to the array common_gene_patterns if there are more than 1 in $pattern & sets the varialbe $pattern = to the empty string
sub add_pattern{
	my($in_pattern,$pattern,$gene_patterns,$numJ) = @_;
	if(($$in_pattern == 1) && ($$pattern =~ /[^ ]+ \([^ ]+\) [^ ]+ /)){
		push(@$gene_patterns,"$$pattern");
		$$in_pattern = 0;
	}
	$$pattern = "";
}

#gets the next gene in genome i/0 after 'gene i' that are contained in genome j/1
sub next_corresponding_gene{
	my($genomes,$cor_genes,$genes,$i) = @_;
	my($gene);
	
	$$i++;
	if($$i < scalar(@$genes)){
		if(exists($$cor_genes{0}{1}{$$genes[$$i]})){
			$gene = $$cor_genes{0}{1}{$$genes[$$i]};
		}else{
			$gene = next_corresponding_gene($genomes,$cor_genes,$genes,$i);
		}
	}else{
		$gene = "nill";
	}
	
	return $gene;
}

#creates a bar graph showing the number of genes in conserved clusters and the number of lost gene between genes in clusters
sub conserved_gene_succession_gif{
	my($output) = @_;
	my(@data);	#holds the data that goes into the bar graph
	my(@length);	#holds the number of genes contained in each of the conserved gene succession clusters.
	my(@lost);	#holds the number of genes that has been lost between two genes in the clusters
	my($i) = 0;
	my($element);
	my($number) = scalar(@common_gene_patterns) + scalar(@re_common_gene_patterns);
	

	foreach $element(@common_gene_patterns){
		while($element =~ /\(([0-9]+)\)/ig){push(@lost,$1);}
		$length[$i] = 1;
		while($element =~ /\(/ig){$length[$i]++;}
		$i++;
	}
	foreach $element(@re_common_gene_patterns){
		while($element =~ /\(([0-9]+)\)/ig){push(@lost,$1);}
		$length[$i] = 1;
		while($element =~ /\(/ig){$length[$i]++;}
		$i++;
	}
	
	@data = (['Max','Mean','Median'],
		[Statistics::max(@lost),sprintf("%.2f",Statistics::mean(@lost)),Statistics::median(@lost)],
		[Statistics::max(@length),sprintf("%.2f",Statistics::mean(@length)),Statistics::median(@length)]
		);
	
	my(@label) = ('Number of genes lost between two genes within a CGSC','Number of genes in a CGSC');
	
	Graph::bars(500,300,\@data,"Conserved gene succession clusters","Number of genes",\@label,$output);
}

sub conserved_gene_succession_overall_gif{
	my($genomes,$output) = @_;
	my(@data);	#holds the data that goes into the bar graph
	my($length) = 0;	#holds the number of genes contained in each of the conserved gene succession clusters.
	my($lost) = 0;	#holds the number of genes that has been lost between two genes in the clusters
	my($element);
	my($number) = scalar(@common_gene_patterns) + scalar(@re_common_gene_patterns);
	
	foreach $element(@common_gene_patterns){
		while($element =~ /\(([0-9]+)\)/ig){$lost += $1;}
		$length++;
		while($element =~ /\(/ig){$length++;}
	}
	foreach $element(@re_common_gene_patterns){
		while($element =~ /\(([0-9]+)\)/ig){$lost += $1;}
		$length++;
		while($element =~ /\(/ig){$length++;}
	}
	my($proLength) = sprintf("%.2f",100*$length/$$genomes[0]{numberGenes});
	my($proLost) = sprintf("%.2f",100*$lost/($$genomes[1]{numberGenes} - $$genomes[0]{numberGenes}));
	
	@data = ([''],
		[$proLength],
		[$proLost]
		);
	
	my(@label) = ("(Number of genes within CGSC/number of genes in reduced genome)*100","(Genes lost within CGSC/genes lost overall)*100");
	
	Graph::bars(500,300,\@data,"Conserved gene succession clusters","%",\@label,$output);
}


#creates a graph showing the placement of gene cluster in the two genomes
sub conserved_gene_succession_genome_gif{
	my($genomes,$cor_genes,$output) = @_;
	my($size) = 500;
	my($gene1,$gene2,$angle,$angle1,$angle2);
	my($index);
	my($r,$g,$b);
	my($im) = GD::SVG::Image->new();

	#create colours
	my($white) = $im->colorAllocate(255,255,255);
	my($blue) = $im->colorAllocate(0,0,255);
	my($red) = $im->colorAllocate(255,102,102);
	my($green) = $im->colorAllocate(102,255,153);
	
	my($black) = $im->colorAllocate(0,0,0);
	my($yellow) = $im->colorAllocate(255,255,0);
	my($genome) = $im->colorAllocate(127,127,255);
	
	$im->transparent($white);
	$im->interlaced('true');

	
	#the four diameters
	my($d1) = int(3*$size/8);
	my($d2) = int(4*$size/8);
	my($d3) = int(6*$size/8);
	my($d4) = int(7*$size/8);
	my($c) = int($size/2);
	
	#creating the 2 basic genomes
	$im->filledArc($size/2,$size/2,$d4,$d4,270,360+270,$genome);
	
	#creates the conserved gene succession clusters in the genome j/1
	foreach my $succession(@common_gene_patterns){
		if( (($gene1,$gene2) = ($succession =~ /^\s*(\S+)\s.+\s(\S+)\s/))){
		
			$angle1 = 360*$$genomes[1]{$gene1}{position}/$$genomes[1]{length} + 270;
			$angle2 = 360*$$genomes[1]{$gene2}{end}/$$genomes[1]{length} + 270;
			$im->filledArc($c,$c,$d4,$d4,$angle1,$angle2,$green);
		}
	}
	
	#creates the reversed conserved gene succession clusters in the genome j/1
	foreach my $succession(@re_common_gene_patterns){
		if( (($gene1,$gene2) = ($succession =~ /\s*(\S+)\s.+\s(\S+)\s/))){
			$angle1 = 360*$$genomes[1]{$gene2}{position}/$$genomes[1]{length} + 270;
			$angle2 = 360*$$genomes[1]{$gene1}{end}/$$genomes[1]{length} + 270;
			
			#determining if its this one that has been reversed - only necessary when the rearrangement has occurred in the reference genome
			if($angle1 > $angle2){
				$angle1 = 360*$$genomes[1]{$gene1}{position}/$$genomes[1]{length} + 270;
				$angle2 = 360*$$genomes[1]{$gene2}{end}/$$genomes[1]{length} + 270;
			}
			
			$im->filledArc($c,$c,$d4,$d4,$angle1,$angle2,$red);
		}
	}
	
	$im->filledArc($c,$c,$d3,$d3,0,360,$white);
	$im->filledArc($c,$c,$d2,$d2,0,360,$genome);

	#creates the conserved gene succession clusters in the genome i/0
	foreach my $succession(@common_gene_patterns){
		if( (($gene1,$gene2) = ($succession =~ /\s*(\S+)\s.+\s(\S+)\s/))){
			$angle1 = 360*$$genomes[0]{$$cor_genes{1}{0}{$gene1}}{position}/$$genomes[0]{length} + 270;
			$angle2 = 360*$$genomes[0]{$$cor_genes{1}{0}{$gene2}}{end}/$$genomes[0]{length} + 270;
			
			$im->filledArc($c,$c,$d2,$d2,$angle1,$angle2,$green);
		}
	}
	
	#creates the reversed conserved gene succession clusters in the genome i/0
	foreach my $succession(@re_common_gene_patterns){
		if( (($gene1,$gene2) = ($succession =~ /\s*(\S+)\s.+\s(\S+)\s/))){
			$angle1 = 360*$$genomes[0]{$$cor_genes{1}{0}{$gene2}}{position}/$$genomes[0]{length} + 270;
			$angle2 = 360*$$genomes[0]{$$cor_genes{1}{0}{$gene1}}{end}/$$genomes[0]{length} + 270;
			
			if($angle2 < $angle1){
				$angle1 = 360*$$genomes[0]{$$cor_genes{1}{0}{$gene1}}{position}/$$genomes[0]{length} + 270;
				$angle2 = 360*$$genomes[0]{$$cor_genes{1}{0}{$gene2}}{end}/$$genomes[0]{length} + 270;
			}	
		
			$im->filledArc($c,$c,$d2,$d2,$angle1,$angle2,$red);
			
		}
	}
	
	$im->filledArc($c,$c,$d1,$d1,0,360,$white);
	
	$im->setThickness(2);
	
	#indicates the position of the origin in the two genomes
	$im->string(GD::SVG::Font->MediumBold,($size - 6*7)/2,3*$size/16-2,"Origin",$blue);
	$im->line($size/2,2*$size/16,$size/2,2*$size/16+20,$blue);
	$im->line($size/2,4*$size/16-15,$size/2,4*$size/16,$blue);
	
	my($Title) = $$genomes[0]{Pure_name}." vs ".$$genomes[1]{Pure_name};#"Conserved genes";
	
	#prints the labels and title
	$im->string(GD::SVG::Font->MediumBold,($size - length($Title)*7)/2,$size/32-10,$Title,$blue);
	$im->line(5,$size-6,15,$size-6,$genome);
	$im->string(GD::SVG::Font->MediumBold,20,$size-5-8,"Genes not in a conserved gene succession cluster",$blue);
	$im->line(5,$size-6+10,15,$size-6+10,$red);
	$im->string(GD::SVG::Font->MediumBold,20,$size-5-8+10,"Reversed conserved gene succession clusters",$blue);
	$im->line(5,$size-6+20,15,$size-6+20,$green);
	$im->string(GD::SVG::Font->MediumBold,20,$size-5-8+20,"Conserved gene succession clusters",$blue);
	
	open(IMG,">$output");
	binmode STDOUT;
	print IMG $im->svg();
	
	close IMG;
}

1;