package Genome_info;

use strict;
use warnings;
use Extract_genome_info;

our(@genomes);		#array-of-hash-of-hash's: array element is the genome name (0=small genomes 1=large genomess)
				#	first hash key:'name' = name of genome
				#			'file' = the file name of the genomes in cgi formalt (GenBank)
				#			'genes' = a string of the genes in the genome (the genes are in the order as they are in the genome) - no paralogous genes found in the genome
				#			'allgenes' = a string of all protein coding genes in the genome
				#			'paralogous' = a string of genes that have a paralog
				#			'genesI' (only for J) = 'genes' but with the gene names from genome I if exists
				#			'genesIonly' (only for J) = genome I's genes in the order of genome J
				#			gene name + 'position' = the position of the gene in the genome
				#			gene name + 'aa' = the genes amino acid sequence.

our(%cor_genes);		#hash-of-hash's: the first and second hash keys are the two genomes (first key from to second key)
				#	third key: 	gene name in first key = gene name in second key
				#			'genes' = the gene list of genome with the gene names of the second key
				#when the 1st and 2nd hash key are the same = looking at paralogous
				#when the 1st and 2nd hash key are genes the value is the e-value observed in the BLAST search

our(%cog);			#hash-of-hast's: first hash key is the genome name
				#	second hash key:'number' = the number of proteins in the genome - the paralogous
				#			cog = the number proteins the that cog
				#			gene name = its coresponding cog
				#			'genes' = genes in the genome
				#OR
				#	first hash key is the broad cog catogarisation
				#		second hash kay: non = a string containing the cog's of that cat
				#				 'number' = the number genes in that cat in genome j

#The subroutine that calles the Extract_genome_info
sub get{
	my($small_genome,$large_genome) = @_;
	
	$genomes[0] = Extract_genome_info::hash($small_genome);
	$genomes[1] = Extract_genome_info::hash($large_genome);
}

#sets the cor_genes hash that contains information on coresponding genes in the two genomes
sub set_cor_genes{
	my($e_value) = @_;
	my($gene);
	my(%blastIJ) = ();
	my(%blastJI) = ();
	
	$cor_genes{0}{1}{genes} = "";
	$cor_genes{1}{0}{genes} = "";
	
	if(! -e $genomes[0]{name}.".fasta"){
		get_fasta(0,\@genomes);
	}
	
	if(! -e $genomes[1]{name}.".fasta"){
		get_fasta(1,\@genomes);
	}
	
	#blast the small genome agains the large genome
	blast(0,1,$genomes[0]{genes},\%blastIJ,$e_value);
	
	#finds the genes in large genome that has been found by blast above
	my(@array) = values(%blastIJ);
	
	#blasts the identified genes in the large genome against the small genome.
	blast(1,0,"@array",\%blastJI,$e_value);

	$genomes[1]{genesI} = "";
	$genomes[1]{genesIonly} = "";
	
	#checkes that the two blasts yearled the same result - and sets the cor_genes hash
	foreach $gene(keys %blastIJ){
		if(exists($blastJI{$blastIJ{$gene}})){
			if($blastJI{$blastIJ{$gene}} eq $gene){
				$cor_genes{0}{1}{$gene} = $blastIJ{$gene};
				$cor_genes{0}{1}{genes} .= "$gene ";
				$cor_genes{1}{0}{$blastIJ{$gene}} = $gene;
				$cor_genes{1}{0}{genes} .= $blastIJ{$gene}." ";
				$cor_genes{$gene}{$blastIJ{$gene}} = $blastIJ{evalue}{$gene}{$blastIJ{$gene}};
				$cor_genes{$blastIJ{$gene}}{$gene} = $blastJI{evalue}{$blastIJ{$gene}}{$gene};
			}
		}
	}
	
	#sets the genesI and genesIonly keys in the 'genomes' hash
	foreach $gene(split(/ /,$genomes[1]{genes})){
		if(exists($cor_genes{1}{0}{$gene})){
			$genomes[1]{genesI} .= $cor_genes{1}{0}{$gene}." ";
			$genomes[1]{genesIonly} .= $cor_genes{1}{0}{$gene}." ";
		}else{
			$genomes[1]{genesI} .= "$gene ";
		}
	}
}

#do an intra BLAST search of the two genomes
#determine paralogous of the non-common genes in the common genes and removing them
sub remove_paralogous{
	my($e_value) = @_;
	my($line,$gene);
	my($read_gene) = 0;
	my(@genes);
	my($hit,$hit_evalue);
	
	my($genome0_fasta) = $genomes[0]{name}.".fasta";
	my($genome0_blast) = $genomes[0]{name}.".blast";
	
	my($genome1_fasta) = $genomes[1]{name}.".fasta";
	my($genome1_blast) = $genomes[1]{name}.".blast";
	
	$genes[0]{common} = "";
	$genes[0]{non_common} = "";
	foreach $gene(split(/ /,$genomes[0]{genes})){
		if(! exists($cor_genes{0}{1}{$gene})){
			$genes[0]{non_common} .= "$gene ";
		}else{
			$genes[0]{common} .= "$gene ";
		}
	}
	
	$genes[1]{common} = "";
	$genes[1]{non_common} = "";
	foreach $gene(split(/ /,$genomes[1]{genes})){
		if(! exists($cor_genes{1}{0}{$gene})){
			$genes[1]{non_common} .= "$gene ";
		}else{
			$genes[1]{common} .= "$gene ";
		}
	}
	
	#intra - BLAST search of reduced genome
	if(! -e $genome0_blast){
		`formatdb -i $genome0_fasta -p T -o T`;
		
		`blastall -p blastp -d $genome0_fasta -i $genome0_fasta -o $genome0_blast`;
	}
	
	unless(open(INPUT,"$genome0_blast")){
		print STDERR "Cant open file $genome0_blast!\n";
	}
	
	$genomes[0]{paralogous} = "";
	#Gets the information on coresponding genes from the created blast file.
	foreach $line(<INPUT>){
		if($line =~ m/^\QQuery=\E (\S+)/){
			$gene = $1;
			$read_gene = 1;
		}elsif(($read_gene == 1) && (($hit,$hit_evalue) = ($line =~ /^(\S+)\s+\d+\s+(\S+)\s+$/))){
			#true when $gene is a non-common gene and the $hit gene is a common gene
			if($hit ne $gene){
				if($genes[0]{non_common} =~ m/$gene/ig){
					if($genes[0]{common} =~ /$hit/){
						if($e_value > $hit_evalue){
							$genomes[0]{genes} =~ s/$gene //;
							$genomes[0]{paralogous} .= "$gene ";
							$cor_genes{0}{0}{$gene} = $hit;
						}
					}
				}
				$read_gene = 0;
			}
		}elsif($read_gene == 1 && $line =~ / No hits found /){
			$read_gene = 0;
		}
	}
	close INPUT;
	
	#intra - BLAST search of reference genome
	if(! -e $genome1_blast){
		`formatdb -i $genome1_fasta -p T -o T`;
	
		`blastall -p blastp -d $genome1_fasta -i $genome1_fasta -o $genome1_blast`;
	}
	
	unless(open(INPUT,"$genome1_blast")){
		print STDERR "Cant open file $genome1_blast!\n";
	}
	
	$genomes[1]{paralogous} = "";
	#Gets the information on coresponding genes from the created blast file.
	foreach $line(<INPUT>){
		if($line =~ /^Query= (\S+)/){
			$gene = $1;
			$read_gene = 1;
		}elsif($read_gene == 1 && (($hit,$hit_evalue) = ($line =~ /^(\S+)\s+\d+\s+(\S+)\s+$/))){
			#true when $gene is a non-common gene and the $hit gene is a common gene
			if($hit ne $gene && ($genes[1]{common} =~ /$hit/) && ($genes[1]{non_common} =~ /$gene/)){
			$hit_evalue =~ s/^e-/1e-/;
				if($e_value > $hit_evalue){
					$genomes[1]{genes} =~ s/$gene //;
					$genomes[1]{paralogous} .= "$gene ";
					$cor_genes{1}{1}{$gene} = $hit;
				}
				$read_gene = 0;
			}
		}elsif($read_gene == 1 && $line =~ / No hits found /){
			$read_gene = 0;
		}
	}
	close INPUT;
	
}

#creates a fasta file ($name) containing the genes in the $genome
sub get_fasta{
	my($name,$genome) = @_;
	my($file) = $$genome[$name]{name}.".fasta";
	
	unless(open(FILE,">$file")){
		print STDERR "Cant open file $file!\n";
	}
	
	foreach my $line(split(/ /,$$genome[$name]{genes})){
		print FILE ">$line\n".$$genome[$name]{$line}{aa}."\n";
	}
	
	close FILE;
}

#Runs a blast search puts the information into a hash (coresponding gene pairs)
sub blast{
	my($firstGenome,$secondGenome,$genes,$hash,$e_value) = @_;
	my($DB_file);
	my($output);
	my($line);
	my($gene);
	my($read_gene) = 0;
	my($blast,$hit,$hit_evalue);
	
	$output = $genomes[0]{directory}.$genomes[$firstGenome]{LOCUS}."_".$genomes[$secondGenome]{LOCUS}.".blast";

	if(! -e $output){
		$blast = $genomes[$firstGenome]{LOCUS}.".fasta";
		
		#creates the fasta file containing the genes that has been founded by the first blast search.
		if($firstGenome == 1){
			get_blast_input($firstGenome,$genes,$blast);
		}else{
			$blast = $genomes[$firstGenome]{name}.".fasta";
		}
		
		$DB_file = $genomes[$secondGenome]{name}.".fasta";
		
	
#####################################################################################################################################################################
		`formatdb -i $DB_file -p T -o T`;
	
		`blastall -p blastp -d $DB_file -i $blast -o $output`;
#####################################################################################################################################################################
		
	}
	
	unless(open(INPUT,"$output")){
		print STDERR "Cant open file $output!\n";
	}
	
	
	#Gets the information on coresponding genes from the created blast file.
	foreach $line(<INPUT>){
		if($line =~ /^Query= ([^\s]+)/){
			$gene = $1;
			$read_gene = 1;
		}elsif($read_gene == 1 && (($hit,$hit_evalue) = ($line =~ /^(\S+)\s+\d+\s+(\S+)\s+$/))){
#		print "$hit,$hit_evalue\n";
			$hit_evalue =~ s/^e-/1e-/;
			if($e_value > $hit_evalue){
				$$hash{$gene} = $hit;
				$$hash{evalue}{$gene}{$hit} = $hit_evalue;
			}
			$read_gene = 0;
		}elsif($read_gene == 1 && $line =~ / No hits found /){
			$read_gene = 0;
		}
	}
	
	close INPUT;
}


#creates a fasta file ($outputFile) containing the genes in $genes
sub get_blast_input{
	my($genome,$genes,$outputFile) = @_;
	
	unless(open(OUTPUT,">$outputFile")){
		print STDERR "Cant open file $outputFile";
	}
	
	foreach my $gene(split(/ /,$genes)){	
		if(exists $genomes[$genome]{$gene}{aa}){
			print OUTPUT ">$gene\n";
			print OUTPUT $genomes[$genome]{$gene}{aa}."\n";
		}
	}
	
	close OUTPUT;
}


#Get the data from each of the .ptt file (contains the COG) and puts the information into a hash called cog
sub get_cog{
	my($file);
	my(@cat) = ('J','A','K','L','B','D','Y','V','T','M','N','Z','W','U','O','C','G','E','F','H','I','P','Q','R','S','-');
	my($s) = 0;
	my(@genome) = (0,1);
	my($i,$k);
	my($gene);
	my($gene_cog);
	my($gene_id);
	my($other_genome) = 1;
	my($temp_cog);
	my($genes) = "";
	my(@temp);

	for($i=0;$i<scalar(@cat);$i++){
		$cog{$genome[0]}{$cat[$i]} = 0;
		$cog{$genome[1]}{$cat[$i]} = 0;
	}	

	#go through each of the genomes
	for($i=0; $i<scalar(@genome);$i++){
		$file = $genomes[$genome[$i]]{name};
		$k=0;
		unless(open(FILE,"$file.ptt")){
			print STDERR "Cant open $file.ptt!\n";
		}
		@temp = split(/ /,$genomes[$genome[$i]]{genes});
		$cog{$genome[$i]}{number} = scalar(@temp);#$genomes[$i]{numberGenes};

		#go through each of the genes in genome $i
		foreach my $line(<FILE>){
			$k++;
			if(($gene,$gene_id,$gene_cog) = ($line =~ /\d+\t\d+\t([^\t]+)\t([^\t]+)\t([^\s]{1})\t/)){
				
				if($gene_id ne '-'){
					$cog{$genome[$i]}{$gene_id} = $gene_cog;
					$genes .= "$gene_id ";
				}else{
					$cog{$genome[$i]}{$gene_cog}++;
					print "no name and no gene id at line $k in $file.ptt\n";
				}
			}
		}
		close FILE;
	}

        #this loop counts the number of genes that are in a each category and if the genes cog is - it checks if the corisponding gene in the other genome has a cog it uses that in sead (assuming that the gene pairs will have the same cog)
	for($i=0;$i<scalar(@genome);$i++){
		foreach $gene(split(/ /,$genomes[$genome[$i]]{genes})){#goes through the genes that are in genome $i - the paralogous are not used
			#true when the genes COG is not -
			if(exists $cog{$genome[$i]}{$gene} && $cog{$genome[$i]}{$gene} ne "-"){
				$cog{$genome[$i]}{$cog{$genome[$i]}{$gene}}++;
			#check if there are a correspoding gene in the other genome if so use its COG
			}elsif(exists($cor_genes{$genome[$i]}{$genome[$other_genome]}{$gene}) && exists($cog{$genome[$other_genome]}{$cor_genes{$genome[$i]}{$genome[$other_genome]}{$gene}})){
				$cog{$genome[$i]}{$cog{$genome[$other_genome]}{$cor_genes{$genome[$i]}{$genome[$other_genome]}{$gene}}}++;
				$cog{$genome[$i]}{$gene} = $cog{$genome[$other_genome]}{$cor_genes{$genome[$i]}{$genome[$other_genome]}{$gene}};
			#true when the nether of the genes has a cog => increase '-'
			}else{
				$cog{$genome[$i]}{'-'}++;
			}
		}
		$other_genome = 0;
	}


	$cog{1}{JKL} = $cog{1}{J} + $cog{1}{A} + $cog{1}{K} + $cog{1}{L} + $cog{1}{B};
	$cog{1}{D_O}  = $cog{1}{D} + $cog{1}{Y} + $cog{1}{V} + $cog{1}{T} + $cog{1}{M} + $cog{1}{N} + $cog{1}{Z} + $cog{1}{W} + $cog{1}{U} + $cog{1}{O};
	$cog{1}{C_Q} = $cog{1}{C} + $cog{1}{G} + $cog{1}{E} + $cog{1}{F} + $cog{1}{H} + $cog{1}{I} + $cog{1}{P} + $cog{1}{Q};
	$cog{1}{RS} = $cog{1}{R} + $cog{1}{S};
	$cog{1}{non} = $cog{1}{'-'};

	$cog{0}{JKL} = $cog{0}{J} + $cog{0}{A} + $cog{0}{K} + $cog{0}{L} + $cog{0}{B};
	$cog{0}{D_O}  = $cog{0}{D} + $cog{0}{Y} + $cog{0}{V} + $cog{0}{T} + $cog{0}{M} + $cog{0}{N} + $cog{0}{Z} + $cog{0}{W} + $cog{0}{U} + $cog{0}{O};
	$cog{0}{C_Q} = $cog{0}{C} + $cog{0}{G} + $cog{0}{E} + $cog{0}{F} + $cog{0}{H} + $cog{0}{I} + $cog{0}{P} + $cog{0}{Q};
	$cog{0}{RS} = $cog{0}{R} + $cog{0}{S};
	$cog{0}{non} = $cog{0}{'-'};

	$cog{JKL}{cat} = "JAKLB";	#Information processing
	$cog{D_O}{cat} = "DYVTMNZWUO";	#Cellular processes and signaling
	$cog{C_Q}{cat} = "CGEFHIPQ";	#Metabolism

	$cog{JKL}{number} = 0;
	$cog{D_O}{number} = 0;
	$cog{C_Q}{number} = 0;

	foreach my $gene(split(/ /,$genomes[1]{genes})){
		if($temp_cog = $cog{1}{$gene}){
		     if($cog{JKL}{cat} =~ /$temp_cog/i){
			$cog{JKL}{number}++;
		     }elsif($cog{D_O}{cat} =~ /$temp_cog/i){
			$cog{D_O}{number}++;
		     }elsif($cog{C_Q}{cat} =~ /$temp_cog/i){
			$cog{C_Q}{number}++;
		     }
		}
	}
	$cog{JKL}{prob} = ($cog{JKL}{number}/$cog{1}{number})**2;
	$cog{D_O}{prob} = ($cog{D_O}{number}/$cog{1}{number})**2;
	$cog{C_Q}{prob} = ($cog{C_Q}{number}/$cog{1}{number})**2;
	$cog{random}{prob} = 1-$cog{JKL}{prob}-$cog{D_O}{prob}-$cog{C_Q}{prob};#-$$cog{RS}{prob}-$$cog{non}{prob};
}

1;
