package Graph;

use strict;
use warnings;
use GD::SVG;

sub xyGraph{
	#(width of svg, height of svg, array of x data, array of y data, min value of x, min value of y, max value of x, max value of y, # of x grids, # of y grids, array of x lables, array of y lables, graph title, x title, y title, output file)
	my($width,$height,$X,$Y,$xMin,$yMin,$xMax,$yMax,$num_xGrid,$num_yGrid,$xLabel,$yLabel,$Title,$xTitle,$yTitle,$output) = @_;
	my($i) = 0;
	
	my($graph) = new GD::SVG::Image($width,$height);
	
	#Defining colours
	my($white) = $graph->colorAllocate(255,255,255); 
	my($not_dots) = $graph->colorAllocate(0,0,155);
	my($dots) = $graph->colorAllocate(255,0,51);
	
	
	#Defining the internal variables
	my($LabelFont) = GD::SVG::gdSmallFont;
	my($LabelFontHeight) = 10;
	my($LabelFontWidth) = 6;
	
	my($TitleFont) = GD::SVG::gdLargeFont;
	my($TitleFontHeight) = 10;
	my($TitleFontWidth) = 8;
	
	
	# find area for write label X, label Y and Title
	my($xLabelWidth) = 2*$LabelFontHeight + 2;
	
	my($yLabelWidth) = 0;
	for ($i=0; $i < scalar(@$yLabel); $i++) {
		if ($yLabelWidth < length($$yLabel[$i])) {
			$yLabelWidth = length($$yLabel[$i]);
		}
	}
	$yLabelWidth *= $LabelFontWidth;           # Y grid
	$yLabelWidth += $LabelFontHeight;           # Y-title in counterclockwise
	
	#deterning the border of the x- and y-axis
	my($xBorder) = 20 + $yLabelWidth + 5;
	my($yBorder) = 20 + $xLabelWidth + $TitleFontHeight + 5;
	
	#find ratio for mapping X,Y into pixel
	my($yScale) = ($height-2*$yBorder)/($yMax - $yMin);
	my($xScale) = ($width-2*$xBorder)/($xMax - $xMin);
	
	
	# draw outer grid lines
	$graph->rectangle($xBorder,$yBorder,$width-1-$xBorder,$height-1-$yBorder,$not_dots);
	
	my($x) = 0;
	my($y) = 0;
	#Drawing long x ticks
	for ($i = 1; $i < $num_xGrid; $i++) {
		$x = ($xMax - $xMin)*$i*$xScale/$num_xGrid + $xBorder;
		$graph->line($x,$yBorder,$x,$height-1-$yBorder,$not_dots);
	}
	#drawing long y ticks
	for ($i = 1; $i < $num_yGrid; $i++) {
		$y = $height - (($yMax - $yMin)*$i*$yScale/$num_yGrid + $yBorder);
		$graph->line($xBorder,$y,$width-1-$xBorder,$y,$not_dots);
	}

	#printing x labels and x title
	for ($i = 0; $i <= $num_xGrid; $i++) {
		$x = ($xMax - $xMin)*$i*$xScale/$num_xGrid + $xBorder - length($$xLabel[$i])*$LabelFontWidth/2;
		$graph->string($LabelFont,$x,$height - ($yBorder-$LabelFontHeight),$$xLabel[$i],$not_dots);
	}
	$x = ($width - length($xTitle)*6)/2;
	$graph->string(GD::SVG::gdMediumBoldFont,$x,$height - 20 - 5,$xTitle,$not_dots);
	
	#printing y labels
	for ($i = 0; $i <= $num_yGrid; $i++) {
		$x = 20 + $LabelFontHeight + ($yLabelWidth - length($$yLabel[$i])*$LabelFontWidth)/2;
		$y = $height - (($yMax - $yMin)*$i*$yScale/$num_yGrid + $yBorder + $LabelFontHeight/2);
		$graph->string($LabelFont,$x,$y,$$yLabel[$i],$not_dots);
	}
	#printing y title
	$x = 0;
	$y = ($height + length($yTitle)*6)/2;
	$graph->stringUp(GD::SVG::gdMediumBoldFont,$x,$y,$yTitle,$not_dots);

	#printing graph title
	$x = ($width - length($Title)*$TitleFontWidth)/2;
	$y = 20;
	$graph->string($TitleFont,$x,$y,$Title,$not_dots);
	$graph->string($TitleFont,$x+1,$y,$Title,$not_dots);

	# ploting x and y data
	for($i=0; $i < scalar(@$X); $i++){
		if(($$X[$i] ne "") && ($$Y[$i] ne "")){
			$x = $xBorder + $$X[$i]*$xScale;
			$y = $height - ($yBorder + $$Y[$i]*$yScale);
			$graph->filledArc($x,$y,4,4,0,360,$dots);
		}
	}

	unless(open(OUT,">$output")){
		print STDERR "Cant open file $output\n";
	}
	
	print OUT $graph->svg;
	close OUT;
}

sub bars{
	my($width,$height,$data,$Title,$yTitle,$legends,$output) = @_;
	
	my($i) = 0;
	my(@yLabel);
	my($yMin) = 0;
	my($num_yGrid) = 5;
	
	push(@yLabel,$yMin);
	$i = MaxArray($data);
	my($yMax) = (int(($i + $i/5)/10 + 1))*10;
	for(my($k)=1;$k<=5;$k++){
		push(@yLabel,int(($k*$yMax)/5));
	}
	
	my($graph) = new GD::SVG::Image($width,$height);
	my(@xLabel) = @{@$data[0]}; 
	
	#Defining colours
	my($white) 	= $graph->colorAllocate(255,255,255); 
	my($not_dots) 	= $graph->colorAllocate(0,0,155);
	my($dots) 	= $graph->colorAllocate(255,0,51);
	
	my(@colour) = ($graph->colorAllocate(255,255,102),$graph->colorAllocate(255,165,0),$graph->colorAllocate(255,51,51),$graph->colorAllocate(153,102,153),$graph->colorAllocate(51,51,153),$graph->colorAllocate(0,153,51),$graph->colorAllocate(105,105,105));
	
	#Defining the internal variables
	my($LabelFont) = GD::SVG::gdSmallFont;
	my($LabelFontHeight) = 10;
	my($LabelFontWidth) = 6;
	
	my($TitleFont) = GD::SVG::gdLargeFont;
	my($TitleFontHeight) = 10;
	my($TitleFontWidth) = 8;
	
	
	# find area for write label X, label Y and Title
	my($xLabelWidth) = 2*$LabelFontHeight + 2;
	
	my($yLabelWidth) = 0;
	for ($i=0; $i < scalar(@yLabel); $i++) {
		if ($yLabelWidth < length($yLabel[$i])) {
			$yLabelWidth = length($yLabel[$i]);
		}
	}
	$yLabelWidth *= $LabelFontWidth;           # Y grid
	$yLabelWidth += $LabelFontHeight;           # Y-title in counterclockwise
	
	#deterning the border of the x- and y-axis
	my($xBorder) = 20 + $yLabelWidth + 5;
	my($yTopBorder) = 20 + $TitleFontHeight + 5;
	my($yBottomBorder) = 20 + ($LabelFontHeight+2)*scalar(@$legends) + 5;
	
	my($yScale) = ($height-($yTopBorder + $yBottomBorder))/($yMax - $yMin);
	my($barWidth) = ($width - 2*$xBorder - 8*(scalar(@{@$data[1]})) - 2*((scalar(@$data)-1))*(scalar(@{@$data[1]})))/(scalar(@{@$data[1]})*(scalar(@$data)));
	
	# draw outer grid lines
	$graph->rectangle($xBorder,$yTopBorder,$width-1-$xBorder,$height-1-$yBottomBorder,$not_dots);
	
	my($x) = 0;
	my($y) = 0;
	#drawing y ticks
	for ($i = 1; $i < $num_yGrid; $i++) {
		$y = $height - (($yMax - $yMin)*$i*$yScale/$num_yGrid + $yBottomBorder);
		$graph->line($xBorder,$y,$xBorder+3,$y,$not_dots);
	}

	#printing x labels
	my $temp = (($width - 2*$xBorder)/(scalar(@xLabel)))/2;
	$i = 1;
	foreach my $label(@xLabel){
		$x = $xBorder + $i*$temp - length($label)*$LabelFontWidth/2;
		$graph->string($LabelFont,$x,$height-$yBottomBorder+2,$label,$not_dots);
		$i+=2;
	}
	
	#printing y labels
	for ($i = 0; $i <= $num_yGrid; $i++) {
		$x = 20 + $LabelFontHeight + ($yLabelWidth - length($yLabel[$i])*$LabelFontWidth)/2;
		$y = $height - (($yMax - $yMin)*$i*$yScale/$num_yGrid + $yBottomBorder + $LabelFontHeight/2);
		$graph->string($LabelFont,$x,$y,$yLabel[$i],$not_dots);
	}
	#printing y title
	$x = 0;
	$y = $yTopBorder + ($height - ($yBottomBorder+$yTopBorder) + 6*length($yTitle))/2;
	$graph->stringUp(GD::SVG::gdMediumBoldFont,$x,$y,$yTitle,$not_dots);

	#printing graph title
	$x = ($width - length($Title)*$TitleFontWidth)/2;
	$y = 15;
	$graph->string($TitleFont,$x,$y,$Title,$not_dots);

	#creating the bars
	my($dataSet) = 0;
	$x = $xBorder + 4;
	for($i=1;$i<scalar(@$data);$i++){
		$x = $xBorder + 12 + $dataSet*($barWidth + 2);
		foreach my $k(@{@$data[$i]}){
			$graph->filledRectangle($x,$height-$yBottomBorder-$k*$yScale-2,$x+$barWidth,$height-$yBottomBorder-2,$colour[$dataSet]);
			$graph->string($LabelFont,$x+$barWidth/2 - ((length(int($k))*$LabelFontWidth)/2),$height-$yBottomBorder-$k*$yScale-12,int($k),$not_dots);
			$x += ($width-2*$xBorder)/(scalar(@{@$data[1]}));
		}
		$dataSet++;
	}
	
	my $yStart = $height;
	$i = scalar(@$legends) - 1;
	
	#print legends
	foreach my $legend(reverse(@$legends)){
		$yStart -= $LabelFontHeight + 2;
		$graph->filledRectangle(5,$yStart+2,10,$yStart+7,$colour[$i]);
		$graph->string($LabelFont,15,$yStart,$legend,$not_dots);
		$i--;
	}
	
	unless(open(OUT,">$output")){
		print STDERR "Cant open file $output\n";
	}
	
	print OUT $graph->svg;
	close OUT;
}

sub mixed{
	my($width,$height,$data,$xMax,$yMax,$Title,$xTitle,$yTitle,$legends,$output) = @_;
	
	my($i) = 0;
	my(@yLabel);
	my($yMin) = 0;
	my($num_yGrid) = 5;
	
	push(@yLabel,$yMin);
	for(my($k)=1;$k<=5;$k++){
		push(@yLabel,int(($k*$yMax)/5));
	}
	
	my($graph) = new GD::SVG::Image($width,$height);
	my(@xLabel) = @{@$data[0]}; 
	
	#Defining colours
	my($white) 	= $graph->colorAllocate(255,255,255); 
	my($not_dots) 	= $graph->colorAllocate(0,0,155);
	my($dots) 	= $graph->colorAllocate(255,0,51);
	
	my(@colour) = ($graph->colorAllocate(72,118,255),$graph->colorAllocate(255,255,102),$graph->colorAllocate(205,38,38));
	
	#Defining the internal variables
	my($LabelFont) = GD::SVG::gdSmallFont;
	my($LabelFontHeight) = 10;
	my($LabelFontWidth) = 6;
	
	my($TitleFont) = GD::SVG::gdLargeFont;
	my($TitleFontHeight) = 10;
	my($TitleFontWidth) = 7;
	
	
	# find area for write label X, label Y and Title
	my($xLabelWidth) = 2*$LabelFontHeight + 2;
	
	my($yLabelWidth) = 0;
	for ($i=0; $i < scalar(@yLabel); $i++) {
		if ($yLabelWidth < length($yLabel[$i])) {
			$yLabelWidth = length($yLabel[$i]);
		}
	}
	$yLabelWidth *= $LabelFontWidth;           # Y grid
	$yLabelWidth += $LabelFontHeight;           # Y-title in counterclockwise
	
	#deterning the border of the x- and y-axis
	my($xBorder) = 20 + $yLabelWidth + 5;
	my($yTopBorder) = 20 + $TitleFontHeight + 5;
	my($yBottomBorder) = 20 + ($LabelFontHeight+2)*scalar(@$legends) + 5;
	
	my($yScale) = ($height-($yTopBorder + $yBottomBorder + 2))/($yMax - $yMin);
	my($barWidth) = ($width - 2*$xBorder - 25)/(scalar(@{@$data[0]}) - 1);
	
	# draw outer grid lines
	$graph->rectangle($xBorder,$yTopBorder,$width-1-$xBorder,$height-1-$yBottomBorder,$not_dots);
	
	my($x) = 0;
	my($y) = 0;
	#drawing y ticks
	for ($i = 1; $i < $num_yGrid; $i++) {
		$y = $height - (($yMax - $yMin)*$i*$yScale/$num_yGrid + $yBottomBorder);
		$graph->line($xBorder,$y,$xBorder+3,$y,$not_dots);
	}
	
	#printing y labels
	for ($i = 0; $i <= $num_yGrid; $i++) {
		$x = 20 + $LabelFontHeight + ($yLabelWidth - length($yLabel[$i])*$LabelFontWidth)/2;
		$y = $height - (($yMax - $yMin)*$i*$yScale/$num_yGrid + $yBottomBorder + $LabelFontHeight/2);
		$graph->string($LabelFont,$x,$y,$yLabel[$i],$not_dots);
	}
	#printing y title
	$x = 0;
	$y = $yTopBorder + ($height - ($yBottomBorder+$yTopBorder) + 6*length($yTitle))/2;
	$graph->stringUp(GD::SVG::gdMediumBoldFont,$x,$y,$yTitle,$not_dots);
	
	#printing graph title
	$x = ($width - length($Title)*$TitleFontWidth)/2;
	$y = 15;
	$graph->string($TitleFont,$x,$y,$Title,$not_dots);

	#creating the bars
	my($dataSet) = 0;
	$x = $xBorder + 4;
	for($i=1;$i<scalar(@$data) -1;$i++){
		$x = $xBorder + 4;
		foreach my $k(@{@$data[$i]}){
			$graph->filledRectangle($x,$height-$yBottomBorder-$k*$yScale-1,$x+$barWidth,$height-$yBottomBorder-1,$colour[$dataSet]);
			$graph->string($LabelFont,$x+$barWidth/2 - ((length(int($k))*$LabelFontWidth)/2),$height-$yBottomBorder-$k*$yScale-12,int($k),$not_dots);
			$graph->rectangle($x,$height-$yBottomBorder-$k*$yScale-1,$x+$barWidth,$height-$yBottomBorder-1,$not_dots);
			$x += $barWidth;
		}
		$dataSet++;
	}
	
	$graph->line($xBorder+6,$height-$yBottomBorder-$$data[3][0]*$yScale-1,$width-$xBorder-6,$height-$yBottomBorder-$$data[3][0]*$yScale-1,$colour[2]);
	
	#printing x labels
	$x = $xBorder + 4;
	foreach my $k(@{@$data[0]}){
		$graph->string($LabelFont,$x+$barWidth/2 - ((length($k)*$LabelFontWidth)/2),$height-$yBottomBorder,$k,$not_dots);
		$x += $barWidth;
	}
	
	$graph->string(GD::SVG::gdMediumBoldFont,$xBorder + ($width - 2*$xBorder)/2 - (length($xTitle)/2)*6,$height-$yBottomBorder+10,$xTitle,$not_dots);
	
	my $yStart = $height;
	$i = scalar(@$legends) - 1;
	foreach my $legend(reverse(@$legends)){
		$yStart -= $LabelFontHeight + 2;
		$graph->filledRectangle(5,$yStart+2,10,$yStart+7,$colour[$i]);
		$graph->string($LabelFont,15,$yStart,$legend,$not_dots);
		$i--;
	}
	
	unless(open(OUT,">$output")){
		print STDERR "Cant open file $output\n";
	}
	
	print OUT $graph->svg;
	close OUT;
}

#returns the max value of a 2D array (excluding the first row)
sub MaxArray{
	my($data) = @_;
	my($max) = 0;
	my($i) = 1;
	for(my($i)=1;$i<scalar(@$data);$i++){
		foreach my $k(@{@$data[$i]}){
			if($max < $k){
				$max = $k;
			}
		}
	}
	return $max;
}


1;