package Options;

use strict;
use warnings;
use Statistics;
use GD::SVG;
use Graph;

use constant PI => 4 * atan2(1,1);


#Creates graphs that showes the distribution of coresponding (&& different) genes in the two genomes
sub placement_lost_retained_genes{
	my($genomes,$cor_genes,$output,$lost) = @_;
	my($size) = 500;
	my($x1,$y1,$x2,$y2);
	my($im) = new GD::SVG::Image($size,$size+20);

	#creation colour
	my($white) = $im->colorAllocate(255,255,255);
	my($blue) = $im->colorAllocate(0,0,255);
	my($red) = $im->colorAllocate(255,0,0);
	my($bluish) = $im->colorAllocate(153,204,255);
	my($green) = $im->colorAllocate(51,255,102);
	my($black) = $im->colorAllocate(0,0,0);
	
	my($smallGenome) = $im->colorAllocate(102,0,102);
	my($largeGenome) = $im->colorAllocate(255,0,0);
	my($Title) = $$genomes[0]{Pure_name}." vs ".$$genomes[1]{Pure_name};	#"Conserved genes";
	
	#printing label(s) and title
	
	$im->string(GD::SVG::gdMediumBoldFont,($size - length($Title)*7)/2,$size/32-10,$Title,$blue);
	if($lost <= 1){
		$im->line(5,$size-5,15,$size-5,$green);
		$im->line(5,$size-6,15,$size-6,$green);
		$im->string(GD::SVG::gdMediumBoldFont,20,$size-5-8,"Genes present in both genomes",$blue);
		
		if($lost == 1){
			$im->line(5,$size-5-10,15,$size-5-10,$blue);
			$im->line(5,$size-6-10,15,$size-6-10,$blue);
			$im->string(GD::SVG::gdMediumBoldFont,20,$size-5-8-10,"Genes not present in both genomes",$blue);
		}
	}else{
		$im->line(5,$size-5,15,$size-5,$blue);
		$im->line(5,$size-6,15,$size-6,$blue);
		$im->string(GD::SVG::gdMediumBoldFont,20,$size-5-8,"Genes not present in both genomes",$blue);
	}
	
	my($angle);
	my($d) = 2*3*$size/16;
	my($x) = $size/2;
	my($y) = $size/2;
	$im->arc($x,$y,$d-1,$d-1,0,360,$smallGenome);
	$im->arc($x,$y,$d,$d,0,360,$smallGenome);
	$im->arc($x,$y,$d+1,$d+1,0,360,$smallGenome);
	my($r1) = $size/8;
	my($r2) = 4*$size/16;
	
	#goes through each of the genes in genome i/0
	foreach my $gene(split(/ /,$$genomes[0]{genes})){
		$angle = 380*$$genomes[0]{$gene}{position}/$$genomes[0]{length} - 90;
		$x1 = $x+$r1*cos($angle*PI/180);
		$y1 = $y+$r1*sin($angle*PI/180);
		$x2 = $x+$r2*cos($angle*PI/180);
		$y2 = $y+$r2*sin($angle*PI/180);
		
		#prints the gene if it is contained in both genomes
		if(exists($$cor_genes{0}{1}{$gene})){
			if($lost <= 1){
				$im->line($x1,$y1,$x2,$y2,$green);
			}
		#print the gene if it isn't contained in both genomes
		}elsif($lost >= 1){
			$im->line($x1,$y1,$x2,$y2,$blue);
		}
	}
	
	$im->line(5,$size-5+10,15,$size-5+10,$smallGenome);
	$im->line(5,$size-6+10,15,$size-6+10,$smallGenome);
	$im->string(GD::SVG::gdMediumBoldFont,20,$size-5-8+10,"The ".$$genomes[0]{SOURCE}." genome",$blue);
	
	
	$d = 2*6*$size/16;
	$im->arc($x,$y,$d-1,$d-1,0,360,$largeGenome);
	$im->arc($x,$y,$d,$d,0,360,$largeGenome);
	$im->arc($x,$y,$d+1,$d+1,0,360,$largeGenome);
	$r1 = 5*$size/16;
	$r2 = 7*$size/16;
	
	#goes through each of the genes in genome j/1
	foreach my $gene(split(/ /,$$genomes[1]{genes})){
		$angle = 380*$$genomes[1]{$gene}{position}/$$genomes[1]{length} - 90;
		$x1 = $x+$r1*cos($angle*PI/180);
		$y1 = $y+$r1*sin($angle*PI/180);
		$x2 = $x+$r2*cos($angle*PI/180);
		$y2 = $y+$r2*sin($angle*PI/180);
		
		#prints the gene if it is contained in both genomes
		if(exists($$cor_genes{1}{0}{$gene})){
			if($lost <= 1){
				$im->line($x1,$y1,$x2,$y2,$green);
			}
		#print the gene if it isn't contained in both genomes
		}elsif($lost >= 1){
			$im->line($x1,$y1,$x2,$y2,$blue);
		}
	}
	

	$im->line(5,$size-5+20,15,$size-5+20,$largeGenome);
	$im->line(5,$size-6+20,15,$size-6+20,$largeGenome);
	$im->string(GD::SVG::gdMediumBoldFont,20,$size-5-8+20,"The ".$$genomes[1]{SOURCE}." genome",$blue);
	
	$im->line(1,1,$size-1,1,$black);
	$im->line(1,1,1,$size+20-1,$black);
	$im->line(1,$size+20-1,$size-1,$size+20-1,$black);
	$im->line($size-1,1,$size-1,$size+20-1,$black);
	
	$im->setThickness(2);
	#shows where the origin of the genomes are.
	$im->line($size/2,2*$size/16,$size/2,3*$size/16+($size/32)-4,$black);#1st line
	$im->line($size/2,3*$size/16+($size/32)+10,$size/2,5*$size/16,$black);#2nd line
	$im->string(GD::SVG::gdMediumBoldFont,($size - 6*7)/2,3*$size/16+($size/32)-4,"Origin",$black);
	
	open(IMG,">$output");
	
	print IMG $im->svg;
	
	close IMG;
	
}

sub none_common_genes{
	my($genomes,$cor_genes,$output) = @_;
	my($gene);
	
	unless(open(FILE,">$output")){
		print STDERR "Cant open file $output!\n";
	}
	print FILE $$genomes[0]{SOURCE}." (".$$genomes[0]{LOCUS}.")\n";
	foreach $gene(split(/ /,$$genomes[0]{allgenes})){
		if(! exists($$cor_genes{0}{1}{$gene})){
			if(exists($$cor_genes{0}{0}{$gene})){
				print FILE "$gene\tparalogou to ".$$cor_genes{0}{0}{$gene}."\n";
			}else{
				print FILE "$gene\n";
			}
		}
	}
	print FILE "\n";
	print FILE $$genomes[1]{SOURCE}." (".$$genomes[1]{LOCUS}.")\n";
	foreach $gene(split(/ /,$$genomes[1]{allgenes})){
		if(! exists($$cor_genes{1}{0}{$gene})){
			if(exists($$cor_genes{1}{1}{$gene})){
				print FILE "$gene\tparalogou to ".$$cor_genes{1}{1}{$gene}."\n";
			}else{
				print FILE "$gene\n";
			}
		}
	}
	
	
}

#create a diagram of the gene arrengment in the two genomes
sub gene_rearrangement{
	my($genomes,$cor_genes,$output) = @_;
	my($geneI,$geneJ);
	my($i) = 0;
	my(@I);
	my(@J);
	
	my($outputTxt);

	
	$outputTxt = $$genomes[0]{LOCUS}."_".$$genomes[1]{LOCUS}.".rearrange.txt";
	
	
	foreach $geneI(split(/ /,$$genomes[0]{genes})){
		if($$genomes[1]{genesIonly} =~ /\b$geneI\b /){
			push(@I,$$genomes[0]{$geneI}{position});
			push(@J,$$genomes[1]{$$cor_genes{0}{1}{$geneI}}{position});
		}
	}
	
	my($width) = 700;
	my($height) = 500;
	my($xMin) = 1;
	my($xMax) = $$genomes[0]{length};
	my($yMin) = 1;
	my($yMax) = $$genomes[1]{length};
	my(@xLabel,@yLabel);
	my($num_xGrid) = 6;
	my($num_yGrid) = 6;
	
	
	my($tempX,$tempY);
	my($tempXtitle,$tempYtitle) = "";
	#creating the x labels
	if($xMax > 10000000){
		$tempX = $xMax/1000000;
		$tempXtitle = "Mbp";
	}elsif($xMax > 10000){
		$tempX = $xMax/1000;
		$tempXtitle = "Kbp";
	}else{
		$tempX = $xMax;
		$tempXtitle = "bp";
	}
	#creating the y labels
	if($yMax > 10000000){
		$tempY = $yMax/1000000;
		$tempYtitle = "Mbp";
	}elsif($yMax > 10000){
		$tempY = $yMax/1000;
		$tempYtitle = "Kbp";
	}else{
		$tempY = $yMax;
		$tempYtitle = "bp";
	}
	my($Title) = "Gene order comparison of whole genomes!";
	my($xTitle) = $$genomes[0]{SOURCE}." ($tempXtitle)";
	my($yTitle) = $$genomes[1]{SOURCE}." ($tempYtitle)";
	push(@xLabel,$xMin);
	push(@yLabel,$yMin);
	for(my($k)=1;$k<=6;$k++){
		push(@xLabel,int(($k*$tempX)/6));
		push(@yLabel,int(($k*$tempY)/6));
	}
	
	Graph::xyGraph($width,$height,\@I,\@J,$xMin,$yMin,$xMax,$yMax,$num_xGrid,$num_yGrid,\@xLabel,\@yLabel,$Title,$xTitle,$yTitle,$output);
	
}


sub common_genes{
	my($genomes,$cor_genes,$output) = @_;
	my(%blast) = ();
	my($file) = $$genomes[0]{directory}.$$genomes[0]{LOCUS}."_".$$genomes[1]{LOCUS}.".blast";#$$genomes[0]{name}.".blast";
	my($line);
	my($gene);
	my($get_blast) = 0;
	my($read_gene) = 0;
	
	unless(open(FILE,">$output")){
		print STDERR "Cant open file $output!\n";
	}
	
	print FILE $$genomes[0]{LOCUS}."\te-value\te-value\t".$$genomes[1]{LOCUS}."\n";
	
	foreach my $i(split(/ /,$$genomes[0]{genes})){
		if(exists($$cor_genes{0}{1}{$i})){
			print FILE $i."\t".$$cor_genes{$i}{$$cor_genes{0}{1}{$i}}."\t".$$cor_genes{$$cor_genes{0}{1}{$i}}{$i}."\t".$$cor_genes{0}{1}{$i}."\n";
		}
	}
	
	close FILE;
}


#creats a bar graph with the distribution of
sub dist_lost_gene_function_gif{
	my($genomes,$cog,$cor_genes,$output) = @_;
	
	my(@cat) = ('J','A','K','L','B','D','Y','V','T','M','N','Z','W','U','O','C','G','E','F','H','I','P','Q','R','S','-');
	my(@data);
	my($gd);
	my($i) = 0;
	my($genes_lost,$proC);
	
	my($pro) = sprintf("%.3f",(($$cog{1}{number}-$$cog{0}{number})/$$cog{1}{number})*100);
	
	foreach my $key(@cat){
		if($$cog{1}{$key} != 0){
			$genes_lost = $$cog{1}{$key} - $$cog{0}{$key};
			if($genes_lost != 0){
				$proC = sprintf("%.3f",($genes_lost/$$cog{1}{$key})*100);
			}else{
				$proC = sprintf("%.3f",0);
			}
			$data[0][$i] = "$key";
			$data[1][$i] = $proC;
			$data[2][$i] = sprintf("%.3f",($$cog{1}{$key}/($$cog{1}{number}-$$cog{0}{number}))*100);
			$data[3][$i] = $pro;
			$i++;
		}
	}
	my(@label) = ('(genes lost in the func.cat./total number of genes lost)*100','(genes lost in the func.cat./genes in the func.cat. in the reference genome)*100','(genes in the reduced genome/gene in the reference genome)*100');
	if(scalar(@{$data[0]}) > 0){
		Graph::mixed(500,300,\@data,100,100,"Distribution of lost genes in functional categories","Functional categories","%",\@label,$output);
	}
}

#Dertermins the number and % of genes lost in each of the cog groups
sub dist_lost_gene_function{
	my($genomes,$cog,$cor_genes,$output) = @_;

	my($num_gene_lost) = 0;
	my($genes_lost) = 0;
	my(@cat) = ('J','A','K','L','B','D','Y','V','T','M','N','Z','W','U','O','C','G','E','F','H','I','P','Q','R','S','-');
	my($proC) = 0;
	my($proA) = 0;

	$num_gene_lost = ($$cog{1}{number} - $$cog{0}{number});

	unless(open(OUT,">$output")){
		print STDERR "cant open $$output\n";
	}

	printf(OUT "%d%8d%8.3f\n",$$cog{0}{number},$$cog{1}{number},((($$cog{1}{number}-$$cog{0}{number})/$$cog{1}{number})*100));
	printf(OUT "%3s%15s%15s%15s\n","COG","genes lost of","% g.l in cat","% g.l overall");
	
	foreach my $key(@cat){
		if(($$cog{0}{$key} != 0) || ($$cog{1}{$key} != 0)){
			$genes_lost = $$cog{1}{$key} - $$cog{0}{$key};
			$proA = $genes_lost/$num_gene_lost;
			
			if($genes_lost != 0){
				$proC = sprintf("%.3f",($genes_lost/$$cog{1}{$key})*100);
			}else{
				$proC = sprintf("%.3f",0);
			}
			
			printf(OUT "%3s%15s%15.3f%15.3f\n",$key,$genes_lost."/".$$cog{1}{$key},$proC,$proA*100);
		}
	}
	close OUT;
}

sub junkDNA{
	my($genomes,$cor_genes,$option,$output) = @_;
	
	my(@junkDNAtrans);
	my(@junkDNAstay);
	my(@junkDNAI,@junkDNAJ,@junkDNAlost,@junkDNAgatherM,@junkDNAgatherL);
	my(@temp);
	my(@genes) = split(/ /,$$genomes[1]{genes});
	my($dist);
	my($i);
	my($temp1,$temp2);
	my($start) = $$genomes[1]{$genes[0]}{end};
	my($lost);
	
	for($i=0;$i< scalar(@genes)-1;$i++){
		$dist = $$genomes[1]{$genes[$i+1]}{position} - $start - 1;
		if($dist < 0){
			$dist = 0;
		}
		if($$genomes[1]{$genes[$i]}{end} < $$genomes[1]{$genes[$i+1]}{end}){
			$start = $$genomes[1]{$genes[$i+1]}{end};
		}else{
			$start = $$genomes[1]{$genes[$i]}{end};
		}
		#when two ajacend genes in the large genomes (J) are pressent in the small (I) genome
		if(exists($$cor_genes{1}{0}{$genes[$i+1]}) && exists($$cor_genes{1}{0}{$genes[$i]})){
			$temp1 = $$cor_genes{1}{0}{$genes[$i]};
			$temp2 = $$cor_genes{1}{0}{$genes[$i+1]};
			if($$genomes[0]{genes} =~ /$temp1 $temp2/ || $$genomes[0]{genes} =~ /$temp2 $temp1/){
				push(@junkDNAstay,$dist);
			}else{
				push(@junkDNAtrans,$dist);
			}
		}elsif(exists($$cor_genes{1}{0}{$genes[$i+1]}) || exists($$cor_genes{1}{0}{$genes[$i]})){
			push(@junkDNAlost,$dist);
		}
	}
	@genes = split(/ /,$$genomes[0]{genes});
	$start = $$genomes[0]{$genes[0]}{end};
	for($i=0;$i<scalar(@genes)-1;$i++){
		$dist = $$genomes[0]{$genes[$i+1]}{position} - $start - 1;
		if($dist < 0){
			$dist = 0;
		}
		if($$genomes[0]{$genes[$i]}{end} < $$genomes[0]{$genes[$i+1]}{end}){
			$start = $$genomes[0]{$genes[$i+1]}{end};
		}else{
			$start = $$genomes[0]{$genes[$i]}{end};
		}
		push(@junkDNAI,$dist);
	}
	
	@genes = split(/ /,$$genomes[1]{genes});
	$start = $$genomes[1]{$genes[0]}{end};
	for($i=0;$i<scalar(@genes)-1;$i++){
		$dist = $$genomes[1]{$genes[$i+1]}{position} - $start - 1;
		if($dist < 0){
			$dist = 0;
		}
		if($$genomes[1]{$genes[$i]}{end} < $$genomes[1]{$genes[$i+1]}{end}){
			$start = $$genomes[1]{$genes[$i+1]}{end};
		}else{
			$start = $$genomes[1]{$genes[$i]}{end};
		}
		
		push(@junkDNAJ,$dist);
	}
	
	@genes = split(/ /,$$genomes[0]{genes});
	
	$start = $$genomes[0]{$genes[0]}{end};
	
	my($test);
	for($i=0;$i<scalar(@genes)-1;$i++){
		$dist = $$genomes[0]{$genes[$i+1]}{position}-$start-1;
		if($dist < 0){
			$dist = 0;
		}
		if($$genomes[0]{$genes[$i]}{end} < $$genomes[0]{$genes[$i+1]}{end}){
			$start = $$genomes[0]{$genes[$i+1]}{end};
		}else{
			$start = $$genomes[0]{$genes[$i]}{end};
		}
		#when two ajacend genes in the large genomes (J) are pressent in the small (I) genome
		if(exists($$cor_genes{0}{1}{$genes[$i+1]}) && exists($$cor_genes{0}{1}{$genes[$i]})){
			
			$temp1 = $$cor_genes{0}{1}{$genes[$i]};
			$temp2 = $$cor_genes{0}{1}{$genes[$i+1]};
			if($$genomes[1]{genes} =~ /($temp1.+$temp2)|($temp2.+$temp1)/){
				if(($test) = ($& =~ /^\w+ (.+) \w+$/)){
					$lost = 1;
					foreach my $gene(split(/ /,$test)){
						if($$cor_genes{1}{0}{$gene}){
							push(@junkDNAgatherM,$dist);
							
							$lost = 0;
							last;
						}
					}
					if($lost == 1){
						push(@junkDNAgatherL,$dist);
						#print OUT $genes[$i]."\t".$genes[$i+1]."\n";
					}
				}
			}
		}
		if(! exists($$cor_genes{0}{1}{$genes[$i]})){
			push(@temp,$dist);
		}
	}

	if($option eq 'j'){
	
		my(@data) = (['Mean','Median'],
				[sprintf("%.2f",Statistics::mean(@junkDNAJ)),Statistics::median(@junkDNAJ)],
				[sprintf("%.2f",Statistics::mean(@junkDNAI)),Statistics::median(@junkDNAI)],
				[sprintf("%.2f",Statistics::mean(@junkDNAstay)),Statistics::median(@junkDNAstay)],
				[sprintf("%.2f",Statistics::mean(@junkDNAtrans)),Statistics::median(@junkDNAtrans)],
				[sprintf("%.2f",Statistics::mean(@junkDNAlost)),Statistics::median(@junkDNAlost)],
				[sprintf("%.2f",Statistics::mean(@junkDNAgatherM)),Statistics::median(@junkDNAgatherM)],
				[sprintf("%.2f",Statistics::mean(@junkDNAgatherL)),Statistics::median(@junkDNAgatherL)]
			);
		
		my($lable1) = "Junk DNA between genes in ".$$genomes[1]{Pure_name};
		$lable1 =~ s/(.{1,68}).*/$1/;
		my($lable2) = "Junk DNA between genes in ".$$genomes[0]{Pure_name};
		$lable2 =~ s/(.{1,68}).*/$1/;
		
		my(@yLabel);
		my($yMin) = 0;
		
		push(@yLabel,$yMin);
		$i = MaxArray(\@data);
		my($yMax) = (int(($i + $i/5)/10 + 1))*10;
		#print "$yMax\n";
		for(my($k)=1;$k<=5;$k++){
			push(@yLabel,int(($k*$yMax)/5));
		}
		
		my(@lable) = ($lable1,$lable2,"Junk DNA between conserved gene succession genes","Junk DNA between gene translocated genes","Junk DNA between genes where one of them has been lost","Junk DNA between genes that has been gathered caused by movement","Junk DNA between genes that has been gathered caused by gene loss");
		
		Graph::bars(500,300,\@data,"Junk DNA","number of base pairs",\@lable,$output);
	}elsif($option eq 'h'){
		unless(open(FILE,">$output")){
			print STDERR "Cant open file $output!\n";
		}
		
		printf(FILE "%6s-%10s-%10s-%7s- %s\n","Number","Max","Mean","Median","Group");
		printf(FILE "%6s-%10s-%10.2f-%7s- %s\n",scalar(@junkDNAI),Statistics::max(@junkDNAI),Statistics::mean(@junkDNAI),Statistics::median(@junkDNAI),$$genomes[0]{Pure_name});
		printf(FILE "%6s-%10s-%10.2f-%7s- %s\n",scalar(@junkDNAJ),Statistics::max(@junkDNAJ),Statistics::mean(@junkDNAJ),Statistics::median(@junkDNAJ),$$genomes[1]{Pure_name});
		printf(FILE "%6s-%10s-%10.2f-%7s- %s\n",scalar(@junkDNAstay),Statistics::max(@junkDNAstay),Statistics::mean(@junkDNAstay),Statistics::median(@junkDNAstay),"Junk DNA between conserved gene succession genes");
		printf(FILE "%6s-%10s-%10.2f-%7s- %s\n",scalar(@junkDNAtrans),Statistics::max(@junkDNAtrans),Statistics::mean(@junkDNAtrans),Statistics::median(@junkDNAtrans),"Junk DNA between gene translocated genes");
		printf(FILE "%6s-%10s-%10.2f-%7s- %s\n",scalar(@junkDNAlost),Statistics::max(@junkDNAlost),Statistics::mean(@junkDNAlost),Statistics::median(@junkDNAlost),"Junk DNA between genes where one of them has been lost");
		printf(FILE "%6s-%10s-%10.2f-%7s- %s\n",scalar(@junkDNAgatherM),Statistics::max(@junkDNAgatherM),Statistics::mean(@junkDNAgatherM),Statistics::median(@junkDNAgatherM),"Junk DNA between genes that has been gathered caused by movement");
		printf(FILE "%6s-%10s-%10.2f-%7s- %s\n",scalar(@junkDNAgatherL),Statistics::max(@junkDNAgatherL),Statistics::mean(@junkDNAgatherL),Statistics::median(@junkDNAgatherL),"Junk DNA between genes that has been gathered caused by gene loss");		
		printf(FILE "%6s %s\n",scalar(@temp),"Number of genes present in the reduced genome but not found in the reference genome");
	
		close FILE;
	}
}

#returns the max value of a 2D array (excluding the first row)
sub MaxArray{
	my($data) = @_;
	my($max) = 0;
	my($i) = 1;
	for(my($i)=1;$i<scalar(@$data);$i++){
		foreach my $k(@{@$data[$i]}){
			if($max < $k){
				$max = $k;
			}
		}
	}
	return $max;
}

1;