package Statistics;

use strict;
use warnings;

sub correlation{
	my($x,$y) = @_;
	
	if(scalar(@$x) != scalar(@$y)){
		print STDERR "There are not an equal number of elements in the two data sets!\n";
		die;
	}
	
	my($n) = scalar(@$x);
	my($sx) = standardDeviation(@$x);
	my($sy) = standardDeviation(@$y);
	my($meanX) = mean(@$x);
	my($meanY) = mean(@$y);
	my($r) = 0;
	
	if($sy != 0 && $sx != 0){
		for(my($i)=0;$i<$n;$i++){
			$r += (($$x[$i]-$meanX)/$sx)*(($$y[$i]-$meanY)/$sy);
		}
		$r = ((1/($n-1))*$r);
	}
	
	return $r;
}

sub max{
	my(@x) = @_;
	my($max) = $x[0];
	foreach my $element(@x){
		if($max < $element){
			$max = $element;
		}
	}
	return $max;
}

sub mean{
	my(@x) = @_;
	
	my($mean) = 0;
	my($n) = scalar(@x);
	if($n > 0){
		foreach my $i(@x){
			$mean += $i;
		}
		
		$mean = $mean/$n;
		
		return $mean;
	}else{
		return 0;
	}
}

sub median{
	my(@x) = @_;
	
	my($median);
	my($n) = scalar(@x);
	if($n > 0){
		my($mod)= $n%2;
		@x = sort{$a <=> $b}@x;     #sorts numerically
		
		if($mod == 1){
			$median = $x[($n+1)/2-1];
		}else{
			$median = ($x[($n+1)/2-1]+$x[($n+1)/2])/2;
		}

		return $median;
	}else{
		return 0;
	}
}

sub standardDeviation{
	my(@x) = @_;
	
	my($mean) = mean(@x);
	my($n) = scalar(@x);
	my($s) = 0;
	
	foreach my $i(@x){
		$s += ($i-$mean)**2;
	}
	
	$s = 1/($n-1)*$s;
	
	$s = ($s)**(1/2);
	
	return $s;
}


1;